import type { ShaderModule } from '@luma.gl/shadertools';
import type { Texture } from '@luma.gl/core';
import type { Bounds } from "../utils/projection-utils.js";
import type { TerrainCover } from "./terrain-cover.js";
/** Module parameters expected by the terrain shader module */
export type TerrainModuleSettings = {
    picking: {
        isActive?: boolean;
    };
    heightMap: Texture | null;
    heightMapBounds?: Bounds | null;
    dummyHeightMap: Texture;
    terrainCover?: TerrainCover | null;
    drawToTerrainHeightMap?: boolean;
    useTerrainHeightMap?: boolean;
    terrainSkipRender?: boolean;
};
/** A model can have one of the following modes */
export declare const TERRAIN_MODE: {
    NONE: number;
    /** A terrain layer rendering encoded ground elevation into the height map */
    WRITE_HEIGHT_MAP: number;
    /** An offset layer reading encoded ground elevation from the height map */
    USE_HEIGHT_MAP: number;
    /** A terrain layer rendering to screen, using the cover fbo overlaid with its own texture */
    USE_COVER: number;
    /** A terrain layer rendering to screen, using the cover fbo as texture */
    USE_COVER_ONLY: number;
    /** Draped layer is rendered into a texture, and never to screen */
    SKIP: number;
};
export declare const terrainModule: ShaderModule<TerrainModuleSettings>;
//# sourceMappingURL=shader-module.d.ts.map